﻿// Accord Statistics Library
// The Accord.NET Framework
// http://accord-framework.net
//
// Copyright © César Souza, 2009-2017
// cesarsouza at gmail.com
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

namespace  Accord.MachineLearning // TODO: Move those classes to the root Accord namespace
{
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    ///   Common interface for parallel algorithms.
    /// </summary>
    /// 
    public interface IParallel : ISupportsCancellation
    {
        /// <summary>
        ///   Gets or sets the parallelization options for this algorithm.
        /// </summary>
        /// 
        ParallelOptions ParallelOptions { get; set; }
    }


    /// <summary>
    ///   Common interface for algorithms that can be canceled
    ///   in the middle of execution.
    /// </summary>
    /// 
    public interface ISupportsCancellation
    {
        /// <summary>
        /// Gets or sets a cancellation token that can be used
        /// to cancel the algorithm while it is running.
        /// </summary>
        /// 
        CancellationToken Token { get; set; }
    }
}
