﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace SharpDX.WPF
{
    public class FPS : INotifyPropertyChanged
    {
        private TimeSpan m_averagingInterval = TimeSpan.FromSeconds(1);

        private List<TimeSpan> m_frames = new List<TimeSpan>();

        private double m_value;

        /// <summary>
        ///
        /// </summary>
        public TimeSpan AveragingInterval
        {
            get { return m_averagingInterval; }
            set
            {
                if (value == m_averagingInterval)
                    return;
                if (value < TimeSpan.FromSeconds(0.1))
                    throw new ArgumentOutOfRangeException();

                m_averagingInterval = value;
                OnPropertyChanged("AveragingInterval");
            }
        }

        /// <summary>
        ///
        /// </summary>
        public double Value
        {
            get { return m_value; }
            private set
            {
                if (value == m_value)
                    return;
                m_value = value;
                OnPropertyChanged("Value");
            }
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="ts"></param>
        public void AddFrame(TimeSpan ts)
        {
            var sec = AveragingInterval;
            var index = m_frames.FindLastIndex(aTS => ts - aTS > sec);
            if (index > -1)
                m_frames.RemoveRange(0, index);
            m_frames.Add(ts);

            UpdateValue();
        }

        /// <summary>
        ///
        /// </summary>
        public void Clear()
        {
            m_frames.Clear();
            UpdateValue();
        }

        /// <summary>
        ///
        /// </summary>
        private void UpdateValue()
        {
            if (m_frames.Count < 2)
            {
                Value = -1;
            }
            else
            {
                var dt = m_frames[m_frames.Count - 1] - m_frames[0];
                Value = dt.Ticks > 100 ? m_frames.Count / dt.TotalSeconds : -1;
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string name)
        {
            var e = PropertyChanged;
            if (e != null)
                e(this, new PropertyChangedEventArgs(name));
        }

        #endregion INotifyPropertyChanged Members
    }
}